#!/usr/bin/env python
# -*- coding:  utf-8 -*-
"""
gottwall.tests
~~~~~~~~~~~~~~~~

Unittests for gottwall

:copyright: (c) 2011 - 2012 by GottWall team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
"""

import unittest

from config import ConfigTestCase
from backends import TCPBackendTestCase, HTTPBackendTestCase, RedisBackendTestCase
from api import APITestCase
from storages import StorageTestCase, RedisStorageTestCase, MemoryStorageTestCase
from utils import UtilsTestCase
from app import ProcessorTestCase


def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(ConfigTestCase))
    suite.addTest(unittest.makeSuite(TCPBackendTestCase))
    suite.addTest(unittest.makeSuite(HTTPBackendTestCase))
    suite.addTest(unittest.makeSuite(RedisBackendTestCase))
    suite.addTest(unittest.makeSuite(APITestCase))
    suite.addTest(unittest.makeSuite(StorageTestCase))
    suite.addTest(unittest.makeSuite(RedisStorageTestCase))
    suite.addTest(unittest.makeSuite(MemoryStorageTestCase))
    suite.addTest(unittest.makeSuite(UtilsTestCase))
    suite.addTest(unittest.makeSuite(ProcessorTestCase))
    return suite

if __name__ == '__main__':
    unittest.main(defaultTest="suite")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
gottwall.processing
~~~~~~~~~~~~~~~~~~~

GottWall dataprocessing

:copyright: (c) 2012 by GottWall team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
:github: http://github.com/Lispython/GottWall
"""

from collections import deque
import tornado.ioloop

from gottwall.settings import PERIODIC_PROCESSOR_TIME, TASKS_CHUNK
from gottwall.log import logger

def process_bucket(processor, app, data, callback=None):
    """Process bucket
    :param processor: :class:`gottwall.processing.PeriodicProcessor` instance
    :param app: :class:`tornado.web.Application` instance
    :param data: bucket data dict
    :param callback: success callback
    """

    result = app.storage.incr(data.pop('project'), **data)
    return result


TASK_TYPES = {"bucket": process_bucket}


class PeriodicProcessor(tornado.ioloop.PeriodicCallback):
    """Periodic data processing
    """

    def __init__(self, app, callback_time=None, io_loop=None, tasks_chunk=None):
        self.application = app
        self.callback_time = callback_time or \
                             app.config.get('PERIODIC_PROCESSOR_TIME', PERIODIC_PROCESSOR_TIME)
        self.io_loop = io_loop or tornado.ioloop.IOLoop.instance()
        self._running = False
        self._timeout = None
        self._deque_chunk_len = app.config.get('TASKS_CHUNK', TASKS_CHUNK)

    def _run(self):
        if not self._running:
            return
        try:
            self.callback()
        except Exception:
            logger.error("Error in periodic callback", exc_info=True)
        self._schedule_next()

    def callback(self):
        """Periodic processor callback

        :param application: application instance
        """
        logger.info("Periodic processor callback")
        try:
            i = 0
            while i < self._deque_chunk_len:
                task_type, data = self.application.tasks.pop()
                self.process_task(task_type, data)
                i += 1
        except IndexError:
            pass
        except Exception, e:
            logger.error(e)

    def process_task(self, task_type, data):
        f = TASK_TYPES.get(task_type, None)

        if f:
            return f(self, self.application, data)
        logger.error("Invalid task type: {0}".format(task_type))


class Tasks(deque):
    """Custom wrapper for deque
    """



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
gottwall.storages
~~~~~~~~~~~~~~~~~

GottWall storages backends

:copyright: (c) 2012 by GottWall Team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
:github: http://github.com/Lispython/gottwall
"""


from logging import getLogger
import tornadoredis
import tornado.gen


from gottwall.utils import get_by_period, MagicDict, get_datetime
from gottwall.settings import STORAGE_SETTINGS_KEY

logger = getLogger("gottwall.storages")


class BaseStorage(object):
    """Base interface for calculation
    """

    def __init__(self, application):
        self._application = application

    @classmethod
    def setup(cls, application):
        """Setup storage to application
        """
        storage = cls(application)
        application.storage = storage
        return storage


    def incr(self, project, name, timestamp, value=1, filters=None, **kwargs):
        """Add count for metric `name` and `filters`

        :param project: project name
        :param name: metric name
        :param filters: list of names
        :param value: increment value
        """
        raise NotImplementedError

    def decr(self, project, name, timestamp, value=-1, filters=None, **kwargs):
        """Sub value from metric `name` in `project`

        :param project: project name
        :param name: metric name
        :param filters: list of names
        :param value: increment value
        """
        raise NotImplementedError

    def slice_data(self, period, from_date, to_date, filter_name, filter_value):
        """Get data by range and filters

        :param period: range periodic
        :param from_date: slice from
        :param to_date: slice to
        :param filter_name: slice for filter
        :param filter_value: slice for filter value
        """
        raise NotImplementedError

    def metrics(self, project):
        """Get metrics
        """
        raise NotImplementedError



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
gottwall.runner
~~~~~~~~~~~~~~~

GottWall runner for standalone applications

:copyright: (c) 2012 by GottWall team, see AUTHORS for more details..
:license: BSD, see LICENSE for more details.
:github: http://github.com/Lispython/gottwall
"""

import sys
import os.path
import time
import signal
from logging import StreamHandler
from optparse import OptionParser, Option

from commandor import Command, Commandor
import tornado.ioloop
from tornado import httpserver
from tornado import autoreload
from tornado.options import _LogFormatter

import gottwall.default_config
from gottwall.config import Config
from gottwall.app import HTTPApplication
from gottwall.log import logger


def configure_logging(logging):
    """Configure logging handler"""
    if logging.upper() not in ['DEBUG', 'INFO', 'CRITICAL', 'WARNING', 'ERROR']:
        return
    print("Setup {0} log level".format(logging))
    logger.setLevel(logging.upper())

    if not logger.handlers:
        channel = StreamHandler()
        channel.setFormatter(_LogFormatter(color=False))
        logger.addHandler(channel)


class Commandor(Commandor):
    """Arguments management utilities
    """

    def run(self, options, args):
        """Prepare env befor command execution

        :param options: options object
        :param args: list or commandor args
        """

        if not options.config:
            self.error("You need specify --config\n")
            super(Commandor, self).run(options, args)
            self.exit()

        config = Config()

        # Load default config
        config.from_module(gottwall.default_config)

        # Rewrite default settings
        config.from_file(options.config)

        return config


class Server(Command):
    """Server commands
    """
    commandor = Commandor


class Start(Command):
    """Run server
    """
    parent = Server

    options = [
        Option("-p", "--port",
               metavar=int,
               default=8889,
               help="Port to run http server"),
        Option("-r", "--reload",
               action="store_true",
               dest="reload",
               default=False,
               help="Auto realod source on changes"),
        Option("-h", "--host",
               metavar="str",
               default="127.0.0.1",
               help="Port for server"),
        Option("-l", "--logging",
               metavar="str",
               default="none",
               help="Log level")]

    def run(self, port, reload, host, logging, **kwargs):
        config = self._commandor_res

        configure_logging(logging)

        application = HTTPApplication(config)
        ioloop = tornado.ioloop.IOLoop.instance()

        application.configure_app(ioloop)

        self.http_server = httpserver.HTTPServer(application)
        self.http_server.listen(port, host)

        if reload:
            self.display("Autoreload enabled")
            autoreload.start(io_loop=ioloop, check_time=100)

        self.display("Server running on 127.0.0.1:{0}".format(port))

        # Init signals handler
        #signal.signal(signal.SIGTERM, self.sig_handler)

        # This will also catch KeyboardInterrupt exception
        #signal.signal(signal.SIGINT, self.sig_handler)

        ioloop.start()

    def sig_handler(self, sig, frame):
        """Catch signal and init callback
        """
        tornado.ioloop.IOLoop.instance().add_callback(self.shutdown)

    def shutdown(self):
        """Stop server and add callback to stop i/o loop"""
        self.display("Shutting down service")

        self.http_server.stop()

        io_loop = tornado.ioloop.IOLoop.instance()
        io_loop.add_timeout(time.time() + 2, io_loop.stop)


class Alembic(Command):
    """SqlAlchemy migration tools
    """
    commandor = Commandor

    # ALBEMIC INTEGRATION
    def __init__(self, *args, **kwargs):
        from alembic.config import CommandLine

        kwargs['parser'] = CommandLine(self.__class__.__name__).parser
        super(Alembic, self).__init__(*args, **kwargs)

    def process(self):
        """Execute command
        """
        self.configure()

        options = self.parse_args()
        return self.run(options, *options.cmd)

    def run(self, options, fn, positional, kwarg):
        from alembic.config import Config as AlembicConfig
        from alembic import util

        config = self._commandor_res

        alembic_cfg = AlembicConfig(os.path.join(os.path.dirname(gottwall.default_config.__file__), 'alembic.ini'))
        alembic_cfg.set_main_option("script_location",
                                   config['ALEMBIC_SCRIPT_LOCATION'])

        alembic_cfg.set_main_option("sqlalchemy.url","{ENGINE}://{USER}:{PASSWORD}@{HOST}:{PORT}/{NAME}".\
                                    format(**config['DATABASE']))

        try:
            fn(alembic_cfg,
               *[getattr(options, k) for k in positional],
               **dict((k, getattr(options, k)) for k in kwarg))
        except util.CommandError, e:
            util.err(str(e))


def main():
    parser = OptionParser(
        usage="%prog [options] <commands> [commands options]",
        add_help_option=False)

    commandor_options = [Option('-c', '--config',
                                metavar="FILE",
                                help='Configuration file')]

    manager = Commandor(parser, sys.argv[1:], commandor_options)
    manager.process()

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json
import datetime
import redis
from random import randint, choice
from stati import RedisClient, Client

stats_client = RedisClient(private_key="my_private_key",
                           public_key="my_public_key",
                           project="test_project",
                           host="10.8.9.8",
                           db=2)
import time
from contextlib import contextmanager

def pretty_time(td):
    """Convert timedelta to pretty

    :param td: timedelta (t2 - t1)
    :return: delta string in seconds or minutes
    """
    if td < 300:
        return "{0} sec".format(td)
    else:
        return "{0} min".format((td / 60.0))

@contextmanager
def measure_time(title, logger=None, **debug_params):
    t1 = time.time()
    print('Started "{0}" at {1}'. format(title, time.ctime(t1)))
    yield
    t2 = time.time()
    print('Finished "{0}" at {1} for the time {2}'.\
          format(title, time.ctime(t2), pretty_time(t2-t1)))


with measure_time("Test stats"):
    for x in xrange(1000):

        stats_client.incr(choice([u"Test"]),
                          timestamp=datetime.datetime(choice([2012]), randint(1, 12), randint(1, 27)),
                          value=1,
                          filters={choice(["views", "orders", "filter1", "filter2"]): choice(["hello", "world", "registered"]),
                                  "clicks": "anonymouse"}
                          )

print("finish")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json
import datetime
from base64 import b64encode
import human_curl as hurl

metric_data = {"name": "my_metric_name",
               "timestamp": datetime.datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%S"),
               "filters": {"views": "registered",
                           "clicks": "anonymouse"},
               "action": "incr",
               "value": 2}

headers = (("content-type", "application/json"),
           ("Authorization", b64encode("{0}:{1}".format("public key", "dwefwefwefwecwef"))))

r = hurl.post("http://127.0.0.1:8889/test_project/api/store",  data=json.dumps(metric_data), headers=headers)

print r.content



#!/usr/bin/env python
# -*- coding:  utf-8 -*-
"""
gottwall.tests
~~~~~~~~~~~~~~~~

Unittests for gottwall

:copyright: (c) 2012 by GottWall team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
"""
import os.path
from base import BaseTestCase

from gottwall.config import Config

class ConfigTestCase(BaseTestCase):
    def test_load(self):
        config = Config()
        filename = os.path.join(os.path.abspath(os.path.dirname(__file__)), "config_file.py")

        config.from_file(filename)

        self.assertTrue(config['SOME_TEST_VAR'], 'some_test_value')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
gottwall.app
~~~~~~~~~~~~

Gottwall main loop

:copyright: (c) 2012 by GottWall team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
:github: http://github.com/Lispython/gottwall
"""
import imp

import os
import importlib
import os.path
import tornado.ioloop
from tornado.web import Application, URLSpec
from tornado.options import define, options
from tornado import httpserver
from tornado import autoreload
from utils import rel
from config import Config, default_settings

from handlers import BaseHandler, DashboardHandler, LoginHandler, HomeHandler,\
     StatsHandler, MetricsHandler, LogoutHandler
from backends import HTTPBackend as HTTPBackendHandler
from processing import PeriodicProcessor, Tasks
from jinja_utils import load_filters, load_extensions, load_globals
from jinja2 import Environment, FileSystemLoader, TemplateNotFound


from sqlalchemy import create_engine
from sqlalchemy.orm import scoped_session, sessionmaker


class HTTPApplication(Application):
    """Base application
    """

    def __init__(self, config):
        self.config = config
        self.db = self.configure_db()
        self.jinja_env = self.configure_env()
        self.data_processor = None
        self.tasks = Tasks()

        params = {"config": self.config,
                  "db": self.db,
                  "env": self.jinja_env}

        self.dirty_handlers = [
            (r"{0}/login".format(self.config['PREFIX']), LoginHandler, params, 'login'),
            (r"{0}/logout".format(self.config['PREFIX']), LogoutHandler, params, 'logout'),
            (r"{0}/dashboard".format(self.config['PREFIX']), DashboardHandler, params, 'dashboard'),
            (r"{0}/(?P<project>.+)/api/stats".format(self.config['PREFIX']), StatsHandler, params, 'api-stats'),
            (r"{0}/(?P<project>.+)/api/metrics".format(self.config['PREFIX']), MetricsHandler, params, 'api-metrics'),
            # Default HTTP backend
            (r"{0}/(?P<project>.+)/api/store".format(self.config['PREFIX']),
             HTTPBackendHandler, params, 'store'),
            (r"{0}/".format(self.config['PREFIX']), HomeHandler, params, 'home'),]

        config['login_url'] = config['PREFIX'] + config['login_url']

        tornado.web.Application.__init__(
            self, [URLSpec(*x) for x in self.dirty_handlers], **config)

    def configure_env(self):
        """Configure template env
        """
        searchpath = list(self.config.get("TEMPLATES_PATH", 'templates'))

        env = Environment(loader=FileSystemLoader(searchpath),
                          auto_reload=self.config.get('TEMPLATE_DEBUG', False),
                          cache_size=self.config.get('JINJA2_CACHE_SIZE', 50),
                          extensions=self.config.get('JINJA2_EXTENSIONS', ()))
        filters = self.config.get('JINJA2_FILTERS', ())
        globals = self.config.get('JINJA2_GLOBALS', ())

        env.filters.update(load_filters(filters))
        env.globals.update(load_globals(globals))

        return env

    def configure_db(self):
        return None
        # engine = create_engine("{ENGINE}://{USER}:{PASSWORD}@{HOST}:{PORT}/{NAME}".\
        #                             format(**self.config['DATABASE']), echo=False)
        # return scoped_session(sessionmaker(bind=engine))

    def configure_app(self, io_loop=None):
        """Configure application backends and storages
        """
        storage = self.configure_storage(self.config['STORAGE'])
        self.configure_backends(self.config['BACKENDS'], io_loop, self.config, storage, self.tasks)

        # Add periodic processing
        self.data_processor = PeriodicProcessor(self, io_loop=io_loop)
        self.data_processor.start()


    def configure_storage(self, storage_path):
        """Configure data storage by path

        :param storage: storage path
        """
        module_path, name = storage_path.rsplit('.', 1)
        try:
            module = importlib.import_module(module_path, name)
            storage = getattr(module, name)
        except (ImportError, AttributeError), e:
            raise Exception("Invalid storage: {0}".format(e))
        return storage.setup(self)

    @staticmethod
    def configure_backends(backends, io_loop, config, storage, tasks):
        """Configture data receive backends

        :param backends: list of backends
        """

        for backend_path in backends:
            module_path, name = backend_path.rsplit('.', 1)
            try:
                backend_module = importlib.import_module(module_path, name)

                backend = getattr(backend_module, name)
            except (ImportError, AttributeError), e:
                raise Exception("Invalid backend: {0}".format(e))
            backend.setup_backend(io_loop, config, storage, tasks)

        return True


if __name__ == "__main__":

    default_settings.update(
        dict(
            site_title=u"GottWall - statistics aggregator",
            login_url="/login",
            template_path=os.path.join(os.path.dirname(__file__), "templates"),
            static_path=os.path.join(os.path.dirname(__file__), "static"),
            xsrf_cookies=False,
            cookie_secret="fkewrlhfwhrfweiurbweuybfrweoubfrowebfioubweoiufbwbeofbowebfbwup2XdTP1o/Vo=",
            autoescape=None))

    default_settings.from_file(options.config)

    application = HTTPApplication(default_settings)
    ioloop = tornado.ioloop.IOLoop.instance()

    application.configure_app(ioloop)

    http_server = httpserver.HTTPServer(application)
    http_server.listen(options.port)
    autoreload.start(io_loop=ioloop, check_time=100)
    ioloop.start()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import json

import socket
import datetime

host = '127.0.0.1'
port = 8887

metric_data = {"name": "my_metric_name",
               "timestamp": datetime.datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%S"),
               "filters": {"views": "registered",
                           "clicks": "anonymouse"},
               "action": "incr",
               "value": 2,
               "project": "test_project",
               "auth": {
                   "private_key": "my_private_key",
                   "public_key": "my_public_key"
                   }}


s = socket.socket()
s.connect((host, port))
f= s.makefile('rwb', bufsize=0)
f.write(json.dumps(metric_data)+'\r\n\r\n')
response= f.read()
f.close()
s.close()

print("finished")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
gottwall.exceptions
~~~~~~~~~~~~~~~~~~~

GottWall core exceptions

:copyright: (c) 2012 by GottWall team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
:github: http://github.com/Lispython/gottwall
"""

class InvalidData(Exception):
    pass

class InvalidParams(Exception):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
gottwall.backends.base
~~~~~~~~~~~~~~~~~~~~~~

Base backends for metric calculation

:copyright: (c) 2012 by GottWall team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
:github: http://github.com/Lispython/gottwall
"""

import json
from tornado import gen

class BaseBackend(object):

    application = None

    @property
    def backend_settings(self):
        """Backend specified settings
        """
        return self.config['BACKENDS'][self.key]

    @property
    def key(self):
        """Settings key for backend
        """
        return "{0}.{1}".format(self.__class__.__module__, self.__class__.__name__)

    @gen.engine
    def process_data(self, project, data, callback=None):
        """Process `data`
        """
        res = False
        if data.get('action', 'incr') == 'incr':
            data.pop('action', None)

            res = (yield gen.Task(self.storage.incr, project, **data))

        if callback:
            callback(res)


    def setup_backend(self, io_loop, config, storage, tasks):
        """Setup backend for application

        :param io_loop: :class:`tornado.ioloop.IOLoop` instance
        :param config: :class:`gottwall.config.Config` instance
        :param storage: :class:`gottwall.storage.Storage` instance
        """
        raise NotImplementedError("You need reimplement setup_backend method")

    def check_key(self, private_key, public_key, project):
        """Check private and public priject keys

        :param private_key: gottwall installation private key
        :param public_key: gottwall project public key
        :param project: project name
        """

        if public_key == self.config['PROJECTS'][project] and \
               private_key == self.config['SECRET_KEY']:
            return True
        return False

    @staticmethod
    def parse_data(data):
        """Parse json bucket to dict

        :param data: string or unicode with data
        """
        return json.loads(data.strip())

    def callback(self, message):
        """Process data from stream

        :param message: stream data
        """
        data = self.parse_data(message)

        if self.check_key(data['auth']['private_key'],
                          data['auth']['public_key'], data['project']):
            self.process_data(data['project'], data)

        return True



#!/usr/bin/env python
# -*- coding:  utf-8 -*-
"""
gottwall.tests
~~~~~~~~~~~~~~~~

Unittests for gottwall

:copyright: (c) 2011 - 2012 by GottWall team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
"""
import os
import datetime
from base64 import b64encode

import json
import random
from base import AsyncBaseTestCase, AsyncHTTPBaseTestCase, RedisTestCaseMixin
from gottwall.app import HTTPApplication
from gottwall.config import Config
import gottwall.default_config
import tornadoredis
from utils import async_test
import tornado.gen
from tornado import ioloop


HOST = os.environ.get('GOTTWALL_REDIS_HOST', "10.8.9.8")


class TCPBackendTestCase(AsyncBaseTestCase):
    def get_app(self):
        config = Config()
        config.from_module(gottwall.default_config)

        config.update({"BACKENDS": {"gottwall.backends.tcpip.TCPIPBackend": {}},
                       "STORAGE": "gottwall.storages.MemoryStorage",
                                 "PROJECTS": {"test_project": "secretkey"},
                                 "PRIVATE_KEY": "myprivatekey"})
        self.app = HTTPApplication(config)
        self.app.configure_app(self.io_loop)

        return self.app

    def test(self):
        print("Base test")



class RedisBackendTestCase(AsyncHTTPBaseTestCase, RedisTestCaseMixin):

    def setUp(self):
        super(RedisBackendTestCase, self).setUp()
        self.client = self._new_client()
        self.client.flushdb()

    def tearDown(self):
        try:
            self.client.connection.disconnect()
            del self.client
        except AttributeError:
            pass
        super(RedisBackendTestCase, self).tearDown()

    def get_new_ioloop(self):
        return ioloop.IOLoop.instance()

    def get_app(self):
        config = Config()
        config.from_module(gottwall.default_config)

        config.update({"BACKENDS": {"gottwall.backends.redis.RedisBackend": {"HOST": HOST}},
                       "STORAGE": "gottwall.storages.MemoryStorage",
                       "REDIS_HOST": HOST,
                       "PROJECTS": {"test_project": "secretkey2"},
                       "SECRET_KEY": "myprivatekey2"})
        app = HTTPApplication(config)
        app.configure_app(self.io_loop)
        return app

    @async_test
    @tornado.gen.engine
    def test_subscribe(self):
        app = self.get_app()
        metric_data = {"name": "redis_metric_{0}".format(random.randint(1, 10)),
                       "timestamp": datetime.datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%S"),
                       "filters": {"views": "anonymouse"},
                       "action": "incr",
                       "value": 2,
                       "type": "        import ipdb; ipdb.set_trace()bucket"}

        client = self.client
        key = "gottwall:{0}:{1}:{2}".format("test_project",
                                            app.config['PROJECTS']['test_project'],
                                            app.config['SECRET_KEY'])

        (yield tornado.gen.Task(client.publish, key,
                                json.dumps(metric_data)))

        from pprint import pprint
        pprint(app.storage._store)


        self.stop()


class HTTPBackendTestCase(AsyncHTTPBaseTestCase):

    def get_app(self):
        config = Config()
        config.from_module(gottwall.default_config)
        config.update({"BACKENDS": [],
                                 "PROJECTS": {"test_project": "secretkey"},
                                 "SECRET_KEY": "myprivatekey"})
        self.app = HTTPApplication(config)
        self.app.configure_app(self.io_loop)
        return self.app

    def test_handler(self):
        app = self.get_app()

        metric_data = {"name": "my_metric_name",
                       "timestamp": datetime.datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%S"),
                       "filters": {"views": "registered",
                                   "clicks": "anonymouse"},
                       "action": "incr",
                       "value": 2}

        auth_value = "GottWall private_key={0}, public_key={1}".format(
            app.config['SECRET_KEY'],
            app.config['PROJECTS']['test_project'])

        authorization = "{0}:{1}".format(app.config['PROJECTS']['test_project'],
                                         app.config['SECRET_KEY'])

        response = self.fetch("/gottwall/test_project/api/store", method="POST",
                              body=json.dumps(metric_data),
                              headers={"content-type": "application/json",
                                       "Authorization": b64encode(authorization)})


        self.assertEquals(response.body, "OK")
        self.assertEquals(response.code, 200)

        auth_value = "GottWall private_key={0}, public_key={1}".format(
            app.config['SECRET_KEY'],
            app.config['PROJECTS']['test_project'])

        response = self.fetch("/gottwall/test_project/api/store", method="POST",
                              body=json.dumps(metric_data),
                              headers={"content-type": "application/json",
                                       "X-GottWall-Auth": auth_value})
        from pprint import pprint
        pprint(app.storage._store)

        self.assertEquals(response.body, "OK")
        self.assertEquals(response.code, 200)

        # Test without authorization

        response = self.fetch("/gottwall/test_project/api/store", method="POST",
                              body=json.dumps(metric_data),
                              headers={"content-type": "application/json"})

        self.assertEquals(response.code, 403)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
gottwall
~~~~~~~~

Simple statistics aggregator


:copyright: (c) 2012 - 2013 by GottWall team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
"""

__all__ = 'get_version',
__author__ = "GottWal team"
__license__ = "BSD, see LICENSE for more details"
__version_info__ = (0, 1, 19)
__build__ = 0x000028
__version__ = ".".join(map(str, __version_info__))
__maintainer__ = "Alexandr Lispython"


def get_version():
    return __version__




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
gottwall.tests.app
~~~~~~~~~~~~~~~~~~

Application test case

:copyright: (c) 2012 by GottWall team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
:github: http://github.com/Lispython/gottwall
"""

from base import AsyncBaseTestCase

import tornado.ioloop
from gottwall.config import Config
import gottwall.default_config

from utils import async_test

from gottwall.app import HTTPApplication
from gottwall.processing import PeriodicProcessor

import random
import datetime

class ProcessorTestCase(AsyncBaseTestCase):

    def get_app(self):
        config = Config()
        config.from_module(gottwall.default_config)
        config.update({"TASKS_CHUNK": 15})
        app = HTTPApplication(config)
        app.configure_app(tornado.ioloop.IOLoop().instance())
        return app

    def get_message(self, i):
        return ("bucket",
                {"name": "processor_metric_{0}".format(i),
                 "timestamp": datetime.datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%S"),
                 "filters": {"filter_name1": "flter_value1"},
                 "action": "incr",
                 "value": 2,
                 "project": "test_processor_project"})

    @async_test
    @tornado.gen.engine
    def test_process(self):
        app = self.get_app()
        processor = app.data_processor

        self.assertTrue(isinstance(processor, PeriodicProcessor))
        self.assertEquals(processor._deque_chunk_len, 15)

        # Add messages
        for x in xrange(20):
            app.tasks.append(self.get_message(x))

        self.assertEquals(len(app.tasks), 20)

        processor.callback()

        self.assertEquals(len(app.tasks), 5)

        processor.callback()

        self.assertEquals(len(app.tasks), 0)

        self.stop()

        for x in xrange(20):
            self.assertTrue("processor_metric_{0}".format(x) in
                            app.storage._store["test_processor_project"])



#!/usr/bin/env python
# -*- coding:  utf-8 -*-
"""
gottwall.tests.api
~~~~~~~~~~~~~~~~~~

Unittests for gottwall api

:copyright: (c) 2012 by GottWall team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
"""

import datetime
import json

from tornado.web import Application, RequestHandler

from gottwall.app import HTTPApplication
from gottwall.config import Config, default_settings
import gottwall.default_config
from base import BaseTestCase, AsyncHTTPBaseTestCase
from gottwall.utils import MagicDict


class APITestCase(AsyncHTTPBaseTestCase):

    def get_app(self):
        config = Config()
        config.from_module(gottwall.default_config)

        config.update({"BACKENDS": []})

        self.app = HTTPApplication(config)
        self.app.configure_app(self.io_loop)
        return self.app

    def test_stats(self):
        storage = self.app.storage
        storage._metrics = {}
        storage._store = MagicDict()

        project_name = "test_project"
        timestamp = datetime.datetime(2012, 12, 21)

        for x in xrange(100):
            storage.incr(project_name, "metric_name",
                         timestamp, filters={"filter1": True,
                                             "filter2": ["web", "iphone", "android"]})


        # Get statistics by weeks
        response = self.fetch("/gottwall/test_project/api/stats?from_date=2012-01-20&to_date=2012-12-31&period=week",
                              method="GET")

        self.assertEquals(response.code, 400)
        self.assertTrue('You need specify name and period' in response.body)

        response = self.fetch("/gottwall/test_project/api/stats?from_date=2012-01&to_date=2012-12-31&period={0}&name=metric_name".format(x),
                              method="GET")

        self.assertEquals(response.code, 400)
        self.assertTrue("Invalid date range params" in response.body)

        for x in ['month', 'day', 'year', 'week', 'minute', 'all']:
            response = self.fetch("/gottwall/test_project/api/stats?from_date=2011-01-20&to_date=2013-12-31&period={0}&name=metric_name".format(x),
                              method="GET")
            self.assertEquals(response.code, 200)

            response_data = json.loads(response.body)
            self.assertTrue(isinstance(response_data['range'], (list, tuple)))
            self.assertEquals(response_data['range'][0][1], 100)


    def test_metrics(self):
        storage = self.app.storage

        storage._metrics = {}
        storage._store = MagicDict()

        filters = {"filter1": "value",
                   "filter2": ["value1", "value2", "value2"]}


        storage.save_metric_meta("test_project", "metric_name",
                                 filters=filters)

        # Get statistics by weeks
        response = self.fetch("/gottwall/test_project/api/metrics",
                              method="GET")

        response_data = json.loads(response.body)

        self.assertEquals(response_data,
                          {"metric_name": {"filter1": [filters['filter1']],
                                           "filter2": ["value1", "value2"]}})
        self.assertEquals(response.code, 200)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
gottwall.log
~~~~~~~~~~~~

Gottwall logging configuration

:copyright: (c) 2013 by GottWall team, see AUTHORS for more details..
:license: BSD, see LICENSE for more details.
:github: http://github.com/GottWall/gottwall
"""
from logging import getLogger

logger = getLogger()



#!/usr/bin/env python
# -*- coding:  utf-8 -*-
"""
gottwall
~~~~~~~~

Realtime statistics aggregation platform

:copyright: (c) 2012 - 2013 by GottWall team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
"""

import sys
import os
try:
    import subprocess
    has_subprocess = True
except:
    has_subprocess = False

from setuptools import Command, setup, find_packages

try:
    readme_content = open(os.path.join(os.path.abspath(
        os.path.dirname(__file__)), "README.rst")).read()
except Exception, e:
    print(e)
    readme_content = __doc__


VERSION = "0.1.19"


class run_audit(Command):
    """Audits source code using PyFlakes for following issues:
        - Names which are used but not defined or used before they are defined.
        - Names which are redefined without having been used.
    """
    description = "Audit source code with PyFlakes"
    user_options = []

    def initialize_options(self):
        all = None

    def finalize_options(self):
        pass

    def run(self):
        try:
            import pyflakes.scripts.pyflakes as flakes
        except ImportError:
            print "Audit requires PyFlakes installed in your system."""
            sys.exit(-1)

        dirs = ['gottwall']
        # Add example directories
        for dir in []:
            dirs.append(os.path.join('examples', dir))
        # TODO: Add test subdirectories
        warns = 0
        for dir in dirs:
            for filename in os.listdir(dir):
                if filename.endswith('.py') and filename != '__init__.py':
                    warns += flakes.checkPath(os.path.join(dir, filename))
        if warns > 0:
            print ("Audit finished with total %d warnings." % warns)
        else:
            print ("No problems found in sourcecode.")


def run_tests():
    from tests import suite
    return suite()

py_ver = sys.version_info

#: Python 2.x?
is_py2 = (py_ver[0] == 2)

#: Python 3.x?
is_py3 = (py_ver[0] == 3)

tests_require = [
    'nose',
    'mock==1.0.1']

install_requires = [
    "redis==2.7.2",
    "tornado==2.4.1",
    "python-dateutil==2.1",
    "tornado-redis==1.0.1",
    "commandor==0.1.1",
    "SQLAlchemy==0.7.9",
    "alembic==0.4.0",
    "Jinja2==2.6"]

if not (is_py3 or (is_py2 and py_ver[1] >= 7)):
    install_requires.append("importlib==1.0.2")

PACKAGE_DATA = []
PROJECT = 'gottwall'
for folder in ['static', 'templates']:
    for root, dirs, files in os.walk(os.path.join(PROJECT, folder)):
        for filename in files:
            PACKAGE_DATA.append("%s/%s" % (root[len(PROJECT) + 1:], filename))

setup(
    name="gottwall",
    version=VERSION,
    description="Realtime statistics aggregation platform",
    long_description=readme_content,
    author="Alex Lispython",
    author_email="alex@obout.ru",
    maintainer="Alexandr Lispython",
    maintainer_email="alex@obout.ru",
    url="https://github.com/gottwall/gottwall",
    packages=find_packages(),
    package_data={'': PACKAGE_DATA},
    entry_points={
        'console_scripts': [
            'gottwall = gottwall.runner:main',
        ]},
    install_requires=install_requires,
    tests_require=tests_require,
    license="BSD",
#    test_suite="nose.collector",
    platforms = ['Linux', 'Mac'],
    classifiers=[
        "Environment :: Web Environment",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX",
        "Topic :: Internet",
        "Topic :: Software Development :: Libraries"
        ],
    cmdclass={'audit': run_audit},
    test_suite = '__main__.run_tests'
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
gottwall.storages.redis
~~~~~~~~~~~~~~~~~~~~~~~

Redis storage for collect statistics

:copyright: (c) 2012 by GottWall Team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
:github: http://github.com/Lispython/gottwall
"""

from logging import getLogger

from itertools import ifilter
import tornadoredis
import tornado.gen
from tornado import gen
from tornado.gen import Task

from gottwall.utils import get_by_period, get_datetime, date_range, OrderedDict
from gottwall.settings import STORAGE_SETTINGS_KEY
from gottwall.storages.base import BaseStorage
from tornadoredis.exceptions import ConnectionError

logger = getLogger()


class Client(tornadoredis.Client):
    def __init__(self, reconnect_callback=None, **kwargs):
        self._reconnect_callback = reconnect_callback
        super(Client, self).__init__(**kwargs)

    def on_disconnect(self):
        if self.subscribed:
            self.subscribed = False
        self._reconnect_callback()

        logger.wart("Reconnect client")


class RedisStorage(BaseStorage):
    """Redis backend to store statistics

    Store counters in redis hash keys
    Key format:
    project_name;mertic_name;period;filter_name1|value1/filter_name2|value2 {
        "2012": 100,
        "2013": 200
        }
    """

    def __init__(self, application):
        super(RedisStorage, self).__init__(application)
        config = self._application.config

        self.client = Client(
            host=config[STORAGE_SETTINGS_KEY].get('HOST', 'localhost'),
            port=int(config[STORAGE_SETTINGS_KEY].get('PORT', 6379)),
            password=config[STORAGE_SETTINGS_KEY].get('PASSWORD', None),
            selected_db=int(config[STORAGE_SETTINGS_KEY].get('DB', 0)))

        self.client._reconnect_callback = self.client.connect

        self.client.connect()

    @gen.engine
    def save_metric_meta(self, pipe, project, name, filters=None, callback=None):
        """Save metric filters

        :param project: project name
        :param name: metric name
        :param filters: metric filters
        """
        if not filters:
            filters = {}

        pipe.sadd(self.get_metrics_key(project), name)

        for f, values in filters.iteritems():
            pipe.sadd(self.get_filters_names_key(project, name), f)

            if isinstance(values, (list, tuple)):
                for value in values:
                    pipe.sadd(self.get_filters_values_key(project, name, f), value)
            else:
                pipe.sadd(self.get_filters_values_key(project, name, f), values)

        if callback:
            callback()


    def get_metrics_key(self, project):
        """Get key for metrics list

        :param project: project name
        """
        return "{0}-metrics".format(project)

    def get_filters_values_key(self, project, metric_name, f):
        """Get key for filter values list

        :param project:
        :param metric_name:
        :param f: filter name
        """
        try:
            return u"{0}-metrics-filter-values:{1}:{2}".format(project, metric_name, f)
        except UnicodeDecodeError:
            return u"{0}-metrics-filter-values:{1}:{2}".format(project, metric_name.decode("utf-8"), f)


    def get_filters_names_key(self, project, metric_name):
        """Get key for metrics filters names

        :param project: project name
        :param metric_name: metric name
        """
        try:
            return u"{0}-metrics-filters:{1}".format(project, metric_name)
        except UnicodeDecodeError:
            return u"{0}-metrics-filters:{1}".format(project, metric_name.decode("utf-8"))


    @tornado.gen.engine
    def incr(self, project, name, timestamp, value=1, filters=None, callback=None, **kwargs):
        """Make incr in redis hash
        """

        pipe = self.client.pipeline()

        for period in self._application.config['PERIODS']:
            if filters:
                for fname, fvalue in filters.iteritems():
                    if not isinstance(fvalue, (list, tuple)):
                        # Make incr for all values
                        fvalue = [fvalue]

                    for fvalue_item in fvalue:
                        pipe.hincrby(self.make_key(
                            project, name, period, {fname: fvalue_item}),
                                     get_by_period(timestamp, period), value)

            pipe.hincrby(self.make_key(project, name, period), get_by_period(timestamp, period), value)

            self.save_metric_meta(pipe, project, name, filters)

        res = (yield gen.Task(pipe.execute))

        if callback:
            callback(res)


    def make_key(self, project, name, period, filters=None):
        """Make key from parameters

        :param project: project name
        :param name: mertric name
        :param period: period prefix
        :param timestamp: timestamp
        :param fname: filter name
        :param fvalue: fil

        """
        parts = [project, name, period]

        if isinstance(filters, dict):
            filters_part = u"/".join(
                [u"{0}|{1}".format(f, filters[f])
                 for f in sorted(filters.keys(), key=lambda x: x) if f])

            if filters_part:
                parts.append(filters_part)

        return u';'.join(parts)


    def filter_by_period(self, data, period, from_date=None, to_date=None):
        """Fulter statistics by `from_date` and `to_date`

        :param from_data:
        :param to_date:
        :return: ifilter generator

        """

        if from_date and to_date:
            new_data = OrderedDict(map(lambda x: (x, 0), date_range(from_date, to_date, period)))
            new_data.update(data)
        else:
            new_data = OrderedDict(data)

        if period == 'all':
            return new_data

        return map(lambda x: (get_by_period(x[0], period), x[1]), sorted(ifilter(lambda x: (True if from_date is None else x[0] >= from_date) and \
                                                                                 (True if to_date is None else x[0] <= to_date), new_data.iteritems()),
                                                                     key=lambda x: x[0]))

    @gen.engine
    def slice_data(self, project, name, period, from_date=None, to_date=None,
                   filter_name=None, filter_value=None, callback=None):

        key = self.make_key(project, name, period,
                                         {filter_name: filter_value})

        items = yield Task(self.client.hgetall, key)

        if callback:
            callback(self.filter_by_period(map(lambda x: (get_datetime(x[0], period), x[1]), items.iteritems()),
                period,
                from_date, to_date))

    @tornado.gen.engine
    def metrics(self, project, callback=None):
        """Return all metrics with filters

        :param project: project name
        :returns: result dict
        """
        client = self.client

        metrics = {}

        for metric_name in (yield gen.Task(client.smembers,
                                       self.get_metrics_key(project))):
            if metric_name not in metrics.keys():
                metrics[metric_name] = {}

            for filter_name in (yield gen.Task(client.smembers,
                                           self.get_filters_names_key(project, metric_name))):
                metrics[metric_name][filter_name] = list(
                    sorted((yield gen.Task(client.smembers,
                                       self.get_filters_values_key(project, metric_name, filter_name)))))

        if callback:
            callback(metrics)





#!/usr/bin/env python
# -*- coding:  utf-8 -*-
"""
gottwall.tests
~~~~~~~~~~~~~~~~

Unittests for gottwall

:copyright: (c) 2012 by GottWall team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
"""

import datetime
import json
from random import choice, randint

from tornado.web import Application
import tornado.gen
import tornado.ioloop

from gottwall.app import HTTPApplication
from gottwall.config import Config, default_settings
import gottwall.default_config
from utils import async_test
import tornadoredis
from tornado.gen import Task

from base import BaseTestCase, AsyncBaseTestCase, RedisTestCaseMixin

from gottwall.storages import MemoryStorage, BaseStorage, RedisStorage
from gottwall.utils import MagicDict


class StorageTestCase(BaseTestCase):

    def test_base_storage(self):
        config = Config()
        config.from_object(gottwall.default_config)

        app = HTTPApplication(config)
        app.configure_storage("gottwall.storages.BaseStorage")

        self.assertTrue(isinstance(app.storage, BaseStorage))

        params = {"project": "project_name",
                  "name": "orders",
                  "timestamp": datetime.datetime.now(),
                  "filters": {"clearing": True,
                              "device": "web"}}

        self.assertRaises(NotImplementedError,
                          app.storage.incr, **params)

        self.assertRaises(NotImplementedError, app.storage.metrics, "project_name")


class MemoryStorageTestCase(AsyncBaseTestCase):

    def get_app(self):
        config = Config()
        config.from_module(gottwall.default_config)
        config.update()
        self.app = HTTPApplication(config)
        self.app.configure_app(tornado.ioloop.IOLoop.instance())

        return self.app

    @async_test
    @tornado.gen.engine
    def test_methods(self):
        storage = MemoryStorage(None)

        self.assertTrue(isinstance(storage, MemoryStorage))
        self.assertTrue(isinstance(storage._store, MagicDict))
        self.assertTrue(isinstance(storage._metrics, dict))

        filters = {"filter1": "value",
                   "filter2": ["value1", "value2", "value2"]}

        res = yield Task(storage.save_metric_meta, "memory_storage", "metric_name",
                         filters=filters)

        self.assertTrue(res)

        self.assertEquals(storage._metrics["memory_storage"]["metric_name"]["filter1"], ["value"])
        self.assertEquals(storage._metrics["memory_storage"]["metric_name"]["filter2"],
                          ["value1", "value2"])


        metrics_res = yield Task(storage.metrics, "memory_storage")

        self.assertEquals(metrics_res,
                          {"metric_name": {"filter1": [filters['filter1']],
                                           "filter2": ["value1", "value2"]}})
        self.stop()


    @async_test
    @tornado.gen.engine
    def test_storage(self):
        app = self.get_app()
        app.configure_storage("gottwall.storages.MemoryStorage")

        storage = app.storage

        self.assertTrue(isinstance(app.storage, MemoryStorage))

        project_name = "test_memory_project"
        timestamp = datetime.datetime(2012, 4, 10, 4, 5)

        for x in xrange(10):
            self.assertTrue((yield Task(storage.incr, project_name, "metric_name",
                                        timestamp, filters={"filter1": True,
                                                            "filter2": ["web", "iphone", "android"]})))

        timestamp2 = datetime.datetime(2012, 1, 10, 3, 43)
        for x in xrange(10):
            self.assertTrue((yield Task(storage.incr, project_name, "metric_name",
                                        timestamp2,
                                        filters={"filter1": True,
                                                 "filter2": ["web", "iphone", "android"]})))

        self.assertEqual(storage._metrics,
                         {'test_memory_project':
                          {'metric_name': {'filter1': [True],
                                           'filter2': ['web',
                                                       'iphone',
                                                       'android']}}})


        for period in ["month", "day", "week", "hour", "minute"]:
            for filter_name, filter_value in (("filter1", True),
                                              ("filter2", "web"),
                                              ("filter2", "iphone"),
                                              (None, None)):
                for x in list((yield Task(storage.slice_data, "test_memory_project",
                                          "metric_name", period,
                                          filter_name=filter_name,
                                          filter_value=filter_value))):
                    self.assertEquals(x[1], 10)


        for period in ["year", "all"]:
            for filter_name, filter_value in (("filter1", True),
                                              ("filter2", "web"),
                                              ("filter2", "iphone"),
                                              (None, None)):
                for x in list((yield Task(storage.slice_data, "test_memory_project",
                                          "metric_name", period,
                                          filter_name=filter_name,
                                          filter_value=filter_value))):
                    self.assertEquals(x[1], 20)
        self.stop()


class RedisStorageTestCase(AsyncBaseTestCase, RedisTestCaseMixin):

    def setUp(self):
        super(RedisStorageTestCase, self).setUp()
        self.client = self._new_client()
        self.client.flushdb()

    def tearDown(self):
        try:
            self.client.flushdb()
            self.client.connection.disconnect()
            del self.client
        except AttributeError:
            pass
        super(RedisStorageTestCase, self).tearDown()

    def get_app(self):
        config = Config()
        config.from_object(gottwall.default_config)

        config.update({"STORAGE_SETTINGS": {
            "HOST": self.redis_settings['HOST']
            }})

        self.app = HTTPApplication(config)
        self.app.configure_app(self.io_loop)

        return self.app

    def get_new_ioloop(self):
        return tornado.ioloop.IOLoop.instance()


    @async_test
    @tornado.gen.engine
    def test_methods(self):
        storage = RedisStorage(self.get_app())

        self.assertEquals(storage.make_key("redis_project_name", "metric_name", "week",
                                           filters={"status": "new",
                                                    "type": "registered"}),
                          "redis_project_name;metric_name;week;status|new/type|registered")


        self.assertTrue(isinstance(storage, RedisStorage))

        filters = {"filter1": "value",
                   "filter2": ["value1", "value2", "value2"]}

        client = self.client

        pipe = client.pipeline()
        storage.save_metric_meta(pipe, "redis_storage_test", "metric_name",
                                 filters=filters)

        (yield Task(pipe.execute))

        metrics = yield Task(
            client.smembers,
            storage.get_metrics_key("redis_storage_test"))

        self.assertEquals(len(metrics), 1)
        self.assertTrue("metric_name" in metrics)

        stored_filters = yield Task(
            client.smembers,
            storage.get_filters_names_key("redis_storage_test", "metric_name"))

        self.assertEquals(len(stored_filters), 2)

        for f, values in filters.items():

            stored_value = yield Task(
                client.smembers,
                storage.get_filters_values_key("redis_storage_test", "metric_name", f))

            self.assertEquals(set(values)
                              if isinstance(values, (list, tuple))
                              else set([values]),
                              stored_value)

        metrics = yield Task(storage.metrics, "redis_storage_test")

        self.assertEquals(metrics,
        {"metric_name": {"filter1": [filters['filter1']],
                         "filter2": ["value1", "value2"]}})
        self.stop()


    @async_test
    @tornado.gen.engine
    def test_metric_meta(self):
        """Get metrics structure
        """
        client = tornadoredis.Client(host=self.redis_settings['HOST'])
        app = self.get_app()
        app.configure_storage("gottwall.storages.RedisStorage")
        storage = app.storage

        pipe = client.pipeline()

        (yield Task(storage.save_metric_meta, pipe, "test_metric_meta_project", "metric_name",
                   filters={"hello": "world",
                            "test": ["value1", "value2"]}))

        self.stop()


    @async_test
    @tornado.gen.engine
    def test_redis_storage(self):
        app = self.get_app()
        app.configure_storage("gottwall.storages.RedisStorage")

        storage = app.storage

        self.assertTrue(isinstance(app.storage, RedisStorage))

        project_name = "test_redis_project"
        timestamp = datetime.datetime(2012, 4, 10, 4, 5)

        for x in xrange(10):
            self.assertTrue((yield Task(storage.incr, project_name, "redis_metric_name",
                             timestamp, filters={"filter1": True,
                                                 "filter2": ["web", "iphone", "android"]})))

        timestamp2 = datetime.datetime(2012, 1, 10, 3, 43)
        for x in xrange(10):
            self.assertTrue((yield Task(storage.incr, project_name, "redis_metric_name",
                         timestamp2,
                         filters={"filter1": True,
                                  "filter2": ["web", "iphone", "android"]})))

        self.assertEqual((yield Task(storage.metrics, project_name)),
                         {'redis_metric_name': {'filter1': ['True'],
                                                'filter2': ['android',
                                                            'iphone',
                                                            'web']}})


        for period in ["month", "day", "week", "hour", "minute"]:
            for filter_name, filter_value in (("filter1", True),
                                              ("filter2", "web"),
                                              ("filter2", "iphone"),
                                              (None, None)):
                for x in list((yield Task(storage.slice_data, "test_redis_project",
                                          "redis_metric_name", period,
                                          filter_name=filter_name,
                                          filter_value=filter_value))):
                    self.assertEquals(int(x[1]), 10)


        for period in ["year", "all"]:
            for filter_name, filter_value in (("filter1", True),
                                              ("filter2", "web"),
                                              ("filter2", "iphone"),
                                              (None, None)):
                for x in list((yield Task(storage.slice_data, "test_redis_project",
                                          "redis_metric_name", period,
                                          filter_name=filter_name,
                                          filter_value=filter_value))):
                    self.assertEquals(int(x[1]), 20)

        self.stop()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
gottwall.utils
~~~~~~~~~~~~~~

Core GottWall utilities

:copyright: (c) 2012 by GottWall team, see AUTHORS for more details.
:license: , see LICENSE for more details.
:github: http://github.com/Lispython/projectname
"""
import os.path
from datetime import datetime, timedelta, date
from urllib2 import parse_http_list

# Backport of OrderedDict() class that runs on Python 2.4, 2.5, 2.6, 2.7 and pypy.
# Passes Python2.7's test suite and incorporates all the latest updates.

try:
    from thread import get_ident as _get_ident
except ImportError:
    from dummy_thread import get_ident as _get_ident

try:
    from _abcoll import KeysView, ValuesView, ItemsView
except ImportError:
    pass


from settings import PROJECT_ROOT, TIMESTAMP_FORMAT, PERIOD_PATTERNS


__all__ = 'rel',


def rel(*args):
    return os.path.join(PROJECT_ROOT, *args)


def timestamp_to_datetime(timestamp, format=TIMESTAMP_FORMAT):
    """Convert `timestamp` to `datetime`

    :param timestamp: str
    """
    if isinstance(timestamp, (str, unicode)):
        return datetime.strptime(timestamp, format)
    return timestamp


def get_by_period(dt, period):
    """Ge"%Y-%m-%dT%H:%M"t period value by timestamp

    :param dt: `datetime.datetime` instance
    :param period: period name
    :returns: str repr of timestamp
    """
    ts = timestamp_to_datetime(dt)

    if period == "week":
        return "{0}-{1}".format(ts.year, ts.isocalendar()[1])
    elif period in PERIOD_PATTERNS:
        return ts.strftime(PERIOD_PATTERNS[period])
    return None


def get_datetime(timestamp, period):
    """Convert str timestamp to datetime object

    Reverse for ``get_by_period``

    :param timestamp: str representation of time
    :param perid: period
    """
    if period == 'week' and isinstance(timestamp, (str, unicode)):
        year, week = timestamp.split("-")
        ret = datetime.strptime("{0}-{1}-1".format(year, week), '%Y-%W-%w')
        if date(int(year), 1, 4).isoweekday() > 4:
            ret -= timedelta(days=7)
        return ret
    if isinstance(timestamp, datetime):
        return timestamp

    elif period in PERIOD_PATTERNS:
        return datetime.strptime(timestamp, PERIOD_PATTERNS[period])
    return None


def parse_dict_header(value):
    """Parse key=value pairs from value list

    :param value: header string
    :return: params dict
    """
    result = {}
    for item in parse_http_list(value):
        if "=" not in item:
            result[item] = None
            continue
        name, value = item.split('=', 1)
        if value[:1] == value[-1:] == '"':
            value = value[1:-1] # strip "
        result[name] = value
    return result


class MagicDict(dict):
    """Dict that create key with value dict if
    if does not exist
    """

    def __getitem__(self, key):
        if key not in self:
            self[key] = self.__class__()
        return dict.__getitem__(self, key)


def date_range(from_date, to_date, period="month"):
    delta =  to_date - from_date

    if period == "hour":
        return [to_date - timedelta(hours=x) for x in range(0, delta.days * 24)]
    elif period == "minute":
        return [to_date - timedelta(minutes=x) for x in range(0, delta.days * 24 * 60)]
    elif period == "day":
        return [to_date - timedelta(days=x) for x in range(0, delta.days)]
    return []


class OrderedDict(dict):
    'Dictionary that remembers insertion order'
    # An inherited dict maps keys to values.
    # The inherited dict provides __getitem__, __len__, __contains__, and get.
    # The remaining methods are order-aware.
    # Big-O running times for all methods are the same as for regular dictionaries.

    # The internal self.__map dictionary maps keys to links in a doubly linked list.
    # The circular doubly linked list starts and ends with a sentinel element.
    # The sentinel element never gets deleted (this simplifies the algorithm).
    # Each link is stored as a list of length three:  [PREV, NEXT, KEY].

    def __init__(self, *args, **kwds):
        '''Initialize an ordered dictionary.  Signature is the same as for
        regular dictionaries, but keyword arguments are not recommended
        because their insertion order is arbitrary.

        '''
        if len(args) > 1:
            raise TypeError('expected at most 1 arguments, got %d' % len(args))
        try:
            self.__root
        except AttributeError:
            self.__root = root = []                     # sentinel node
            root[:] = [root, root, None]
            self.__map = {}
        self.__update(*args, **kwds)

    def __setitem__(self, key, value, dict_setitem=dict.__setitem__):
        'od.__setitem__(i, y) <==> od[i]=y'
        # Setting a new item creates a new link which goes at the end of the linked
        # list, and the inherited dictionary is updated with the new key/value pair.
        if key not in self:
            root = self.__root
            last = root[0]
            last[1] = root[0] = self.__map[key] = [last, root, key]
        dict_setitem(self, key, value)

    def __delitem__(self, key, dict_delitem=dict.__delitem__):
        'od.__delitem__(y) <==> del od[y]'
        # Deleting an existing item uses self.__map to find the link which is
        # then removed by updating the links in the predecessor and successor nodes.
        dict_delitem(self, key)
        link_prev, link_next, key = self.__map.pop(key)
        link_prev[1] = link_next
        link_next[0] = link_prev

    def __iter__(self):
        'od.__iter__() <==> iter(od)'
        root = self.__root
        curr = root[1]
        while curr is not root:
            yield curr[2]
            curr = curr[1]

    def __reversed__(self):
        'od.__reversed__() <==> reversed(od)'
        root = self.__root
        curr = root[0]
        while curr is not root:
            yield curr[2]
            curr = curr[0]

    def clear(self):
        'od.clear() -> None.  Remove all items from od.'
        try:
            for node in self.__map.itervalues():
                del node[:]
            root = self.__root
            root[:] = [root, root, None]
            self.__map.clear()
        except AttributeError:
            pass
        dict.clear(self)

    def popitem(self, last=True):
        '''od.popitem() -> (k, v), return and remove a (key, value) pair.
        Pairs are returned in LIFO order if last is true or FIFO order if false.

        '''
        if not self:
            raise KeyError('dictionary is empty')
        root = self.__root
        if last:
            link = root[0]
            link_prev = link[0]
            link_prev[1] = root
            root[0] = link_prev
        else:
            link = root[1]
            link_next = link[1]
            root[1] = link_next
            link_next[0] = root
        key = link[2]
        del self.__map[key]
        value = dict.pop(self, key)
        return key, value

    # -- the following methods do not depend on the internal structure --

    def keys(self):
        'od.keys() -> list of keys in od'
        return list(self)

    def values(self):
        'od.values() -> list of values in od'
        return [self[key] for key in self]

    def items(self):
        'od.items() -> list of (key, value) pairs in od'
        return [(key, self[key]) for key in self]

    def iterkeys(self):
        'od.iterkeys() -> an iterator over the keys in od'
        return iter(self)

    def itervalues(self):
        'od.itervalues -> an iterator over the values in od'
        for k in self:
            yield self[k]

    def iteritems(self):
        'od.iteritems -> an iterator over the (key, value) items in od'
        for k in self:
            yield (k, self[k])

    def update(*args, **kwds):
        '''od.update(E, **F) -> None.  Update od from dict/iterable E and F.

        If E is a dict instance, does:           for k in E: od[k] = E[k]
        If E has a .keys() method, does:         for k in E.keys(): od[k] = E[k]
        Or if E is an iterable of items, does:   for k, v in E: od[k] = v
        In either case, this is followed by:     for k, v in F.items(): od[k] = v

        '''
        if len(args) > 2:
            raise TypeError('update() takes at most 2 positional '
                            'arguments (%d given)' % (len(args),))
        elif not args:
            raise TypeError('update() takes at least 1 argument (0 given)')
        self = args[0]
        # Make progressively weaker assumptions about "other"
        other = ()
        if len(args) == 2:
            other = args[1]
        if isinstance(other, dict):
            for key in other:
                self[key] = other[key]
        elif hasattr(other, 'keys'):
            for key in other.keys():
                self[key] = other[key]
        else:
            for key, value in other:
                self[key] = value
        for key, value in kwds.items():
            self[key] = value

    __update = update  # let subclasses override update without breaking __init__

    __marker = object()

    def pop(self, key, default=__marker):
        '''od.pop(k[,d]) -> v, remove specified key and return the corresponding value.
        If key is not found, d is returned if given, otherwise KeyError is raised.

        '''
        if key in self:
            result = self[key]
            del self[key]
            return result
        if default is self.__marker:
            raise KeyError(key)
        return default

    def setdefault(self, key, default=None):
        'od.setdefault(k[,d]) -> od.get(k,d), also set od[k]=d if k not in od'
        if key in self:
            return self[key]
        self[key] = default
        return default

    def __repr__(self, _repr_running={}):
        'od.__repr__() <==> repr(od)'
        call_key = id(self), _get_ident()
        if call_key in _repr_running:
            return '...'
        _repr_running[call_key] = 1
        try:
            if not self:
                return '%s()' % (self.__class__.__name__,)
            return '%s(%r)' % (self.__class__.__name__, self.items())
        finally:
            del _repr_running[call_key]

    def __reduce__(self):
        'Return state information for pickling'
        items = [[k, self[k]] for k in self]
        inst_dict = vars(self).copy()
        for k in vars(OrderedDict()):
            inst_dict.pop(k, None)
        if inst_dict:
            return (self.__class__, (items,), inst_dict)
        return self.__class__, (items,)

    def copy(self):
        'od.copy() -> a shallow copy of od'
        return self.__class__(self)

    @classmethod
    def fromkeys(cls, iterable, value=None):
        '''OD.fromkeys(S[, v]) -> New ordered dictionary with keys from S
        and values equal to v (which defaults to None).

        '''
        d = cls()
        for key in iterable:
            d[key] = value
        return d

    def __eq__(self, other):
        '''od.__eq__(y) <==> od==y.  Comparison to another OD is order-sensitive
        while comparison to a regular mapping is order-insensitive.

        '''
        if isinstance(other, OrderedDict):
            return len(self)==len(other) and self.items() == other.items()
        return dict.__eq__(self, other)

    def __ne__(self, other):
        return not self == other

    # -- the following methods are only used in Python 2.7 --

    def viewkeys(self):
        "od.viewkeys() -> a set-like object providing a view on od's keys"
        return KeysView(self)

    def viewvalues(self):
        "od.viewvalues() -> an object providing a view on od's values"
        return ValuesView(self)

    def viewitems(self):
        "od.viewitems() -> a set-like object providing a view on od's items"
        return ItemsView(self)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
gottwall.settings
~~~~~~~~~~~~~~~~~

GottWall settings

:copyright: (c) 2012 by GottWall Team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
:github: http://github.com/Lispython/gottwall
"""

import os.path

__all__ = 'PROJECT_ROOT',

PROJECT_ROOT = os.path.abspath(os.path.dirname(__file__))


BACKENDS = []


PERIODS = [
    "week",
    "day",
    "year",
    "month",
    "hour",
    "all",
    "minute"
    ]

TIMESTAMP_FORMAT = "%Y-%m-%dT%H:%M:%S"


PERIOD_PATTERNS = {
    "day": "%Y-%m-%d",
    "year": "%Y",
    "month": "%Y-%m",
    "hour": "%Y-%m-%dT%H",
    "minute": "%Y-%m-%dT%H:%M",
    "all": "all"}


STORAGE_SETTINGS_KEY = "STORAGE_SETTINGS"


DATE_FILTER_FORMAT = "%Y-%m-%d"

PERIODIC_PROCESSOR_TIME = 1000*60*1

TASKS_CHUNK = 20



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os.path

BACKENDS = {}

STORAGE = "gottwall.storages.MemoryStorage"
STORAGE_SETTINGS = {}

PROJECTS = {}

USERS = []

PERIODS = [
    "week",
    "day",
    "year",
    "month",
    "hour",
    "all",
    "minute"]


TEMPLATES_PATH = [
    os.path.join(os.path.dirname(__file__), "templates")
    ]

static_path = os.path.join(os.path.dirname(__file__), "static")
static_url_prefix = "/static/"

login_url = '/login'

site_title=u"GottWall - statistics aggregator"

ALEMBIC_SCRIPT_LOCATION = 'gottwall:migrations'


DATABASE = {
    "ENGINE": "postgresql",
    "NAME": "gottwall",
    "USER": "",
    "PASSWORD": "",
    "HOST": "",
    "PORT": ""}

PREFIX = '/gottwall'

# Run every minute
PERIODIC_PROCESSOR_TIME = 1000*60*5

cookie_secret = 'cookie_secret'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
gottwall.backends.redis
~~~~~~~~~~~~~~~~~~~~~~~

Redis pub/sub backend

:copyright: (c) 2012 by GottWall team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
:github: http://github.com/Lispython/gottwall
"""

import json
from logging import getLogger
from tornado.web import HTTPError
import tornado.ioloop
from tornado import gen

from gottwall.backends.base import BaseBackend
from gottwall.handlers import BaseHandler

import tornadoredis
from tornadoredis.exceptions import ConnectionError

logger = getLogger()


class Client(tornadoredis.Client):
    def __init__(self, reconnect_callback=None, **kwargs):
        self._reconnect_callback = reconnect_callback
        super(Client, self).__init__(**kwargs)

    def on_disconnect(self):
        if self.subscribed:
            self.subscribed = False
        self._reconnect_callback()
        logger.warn("Reconnect client")

    def connect(self):
        if not self.connection.connected():
            pool = self._connection_pool
            if pool:
                old_conn = self.connection
                self.connection = pool.get_connection(event_handler_proxy=self)
                self.connection.ready_callbacks = old_conn.ready_callbacks
            else:
                self.connection.connect()


class RedisBackend(BaseBackend):

    def __init__(self, io_loop, config, storage, tasks):
        self.io_loop = io_loop
        self.config = config
        self.storage = storage
        self.client = None
        self.tasks = tasks

    def get_redis_client(self):
        """Get redis client instance
        """

        client = Client(
            #connection_pool=connection_pool,
            host=self.backend_settings.get('HOST', 'localhost'),
            port=int(self.backend_settings.get('PORT', 6379)),
            io_loop=self.io_loop,
            password=self.backend_settings.get('PASSWORD', None),
            selected_db=int(self.backend_settings.get('DB', 0)),
            reconnect_callback=self.listen)

        return client

    def configure_client(self):
        """Make redis client instance
        """
        self.client = self.get_redis_client()
        return self.client

    @classmethod
    def setup_backend(cls, io_loop, config, storage, tasks):
        """Setup data handler

        :param io_loo: :class:`tornado.ioloop.IOLoop` object
        :param confi: :class:`gottwall.config.Config` object
        :param storage: :class:`gottwall.storage.Storage` object
        """
        backend = cls(io_loop, config, storage, tasks)
        backend.configure_client()
        backend.listen()

    @gen.engine
    def listen(self):
        """Listen socket

        :param client: redis client
        """
        self.client.connect()

        yield tornado.gen.Task(self.client.psubscribe,
                               "{0}:*".format(self.backend_settings.get('CHANNEL', 'gottwall')))
        self.client.listen(self.callback)


    def callback(self, message):

        if message.body == 1:
            # handshake
            return

        try:
            project, public_key, private_key = self.parse_channel(message.channel)
        except ValueError:
            logger.warn("Invalid channel credentails")
            return

        data = self.parse_data(message.body.strip())
        message_type = data.get('type', None)

        if message_type == 'notification':
            # Load data from key to deque for project
            self.load_buckets(project)
        elif message_type == 'bucket':
            self.process_data(project, self.parse_data(data))

        return True

    def parse_channel(self, channel):
        """Parser private, public keys from channel name

        :param message: message object
        :return: tuple for (project, public, private)
        """
        return channel.split(":")[1:]

    def bucket_key(self, project):
        """Build bucket key

        :param project: project name
        """
        return "{0}:{1}:{2}".format(self.backend_settings.get('CHANNEL', 'gottwall'),
                                    project, self.config['PROJECTS'][project])

    @gen.engine
    def load_buckets(self, project):
        """Load data from key to application deque

        :param project: project name
        """
        client = self.get_redis_client()
        key = self.bucket_key(project)
        length = (yield gen.Task(client.scard, key))

        # Max load elements at once
        i = min(self.backend_settings.get("MAX_LOADING", 20), length)

        while i > 0:
            raw_data = (yield gen.Task(client.spop, key))

            if not raw_data:
                break
            try:
                self.process_data(project, self.parse_data(raw_data))
            except Exception, e:
                logger.warning(e)
                print(e)

            i -= 1

